@@ -26,7 +26,7 @@ import torch.nn as nn
 
 
 class AbstractRunner(ABC):
-    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, save_model=False, supervision_npy_file_suffix='_supervision.npy', macro_supervision_npy_file_suffix='_macro_supervision.npy', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', shortcut_features_names_saving_suffix = '_shortcut_features_names.npy', shortcut_features_saving_suffix='_shortcut_features.npy',returns_pkl_file_suffix='_returns.pkl',  local_root_directory='../data/', user = 'napoleon', n_start = 252, maximum_precision = False, shortcut_features = False, transaction_costs = None):
+    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, save_model=False, supervision_npy_file_suffix='_supervision.npy', macro_supervision_npy_file_suffix='_macro_supervision.npy', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', shortcut_features_names_saving_suffix = '_shortcut_features_names.npy', shortcut_features_saving_suffix='_shortcut_features.npy',returns_pkl_file_suffix='_returns.pkl',  local_root_directory='../data/', user = 'napoleon', n_start = 252, maximum_precision = False, shortcut_features = False, transaction_costs = None, crypto_ticker = None):
         super().__init__()
         self.local_root_directory=local_root_directory
         self.supervision_npy_file_suffix=supervision_npy_file_suffix
@@ -35,7 +35,6 @@ class AbstractRunner(ABC):
         self.features_names_saving_suffix=features_names_saving_suffix
 
         self.returns_pkl_file_suffix=returns_pkl_file_suffix
-        self.returns_pkl_file_name = running_date.strftime('%d_%b_%Y')+returns_pkl_file_suffix
 
         self.shortcut_features_saving_suffix = shortcut_features_saving_suffix
         self.shortcut_features_names_saving_suffix = shortcut_features_names_saving_suffix
@@ -50,6 +49,15 @@ class AbstractRunner(ABC):
         self.maximum_precision = maximum_precision
         self.transaction_costs = transaction_costs
 
+
+        self.crypto_ticker = crypto_ticker
+        if self.crypto_ticker is not None:
+            self.returns_pkl_file_name =  self.crypto_ticker + '_' +self.starting_date.strftime('%d_%b_%Y') + '_' + self.running_date.strftime('%d_%b_%Y') + self.returns_pkl_file_suffix
+        else :
+            self.returns_pkl_file_name = self.running_date.strftime('%d_%b_%Y') + self.returns_pkl_file_suffix
+
+
+
     @abstractmethod
     def runTrial(self,saver, seed, sup, layers, epochs, n_past_features, n, s, whole_history, advance_feature, advance_signal,
                  normalize, activation_string, convolution, lr_start, lr_type, low_bound, up_bound):
