@@ -34,7 +34,7 @@ def unjsonize_mapping_dataframe(mapping_df):
 
 class ParallelRunResultAnalyzer():
 
-    def __init__(self,frequence = None, underlying = None, algo = None, starting_date = None, running_date = None, drop_token='', dropbox_backup=True, local_root_directory='../data',user='napoleon',  db_path_suffix = None, max_table_number = 200):
+    def __init__(self,frequence = None, underlying = None, algo = None, starting_date = None, running_date = None, drop_token='', dropbox_backup=True, local_root_directory='../data',user='napoleon',  db_path_name = None, max_table_number = 200):
         self.local_root_directory = local_root_directory
         self.user = user
 
@@ -44,10 +44,9 @@ class ParallelRunResultAnalyzer():
         self.starting_date=starting_date
         self.running_date=running_date
 
-        self.db_path_suffix = db_path_suffix
-        self.filename =  launcher_utility.get_database_name(user, frequence, underlying, algo, starting_date, running_date, db_path_suffix)
+        self.db_path_name = db_path_name
 
-        self.db_path = self.local_root_directory + self.filename
+        self.db_path = self.local_root_directory + self.db_path_name
         self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token,dropbox_backup=dropbox_backup)
         self.max_table_number = max_table_number
 
