@@ -313,8 +313,8 @@ class RealTimeSignalEnsemblingParalleLauncher():
         self.day_calib_signals_list = list(self.day_calib_signals_list )
         self.day_calib_signals_list.sort()
 
-        #self.selected_indice = None
-        self.selected_indice = 'slope_lsTrue13852729'
+        self.selected_indice = None
+        #self.selected_indice = 'slope_lsTrue13852729'
 
         self.selected_counter = 0
         sig_counter = 0
