@@ -15,6 +15,7 @@ from binance.client import Client
 import numpy as np
 from napoleontoolbox.connector import napoleon_s3_connector
 import pandas as pd
+from collections import defaultdict
 
 from datetime import datetime, timedelta
 
@@ -248,8 +249,8 @@ def pair_handling_message(fetcher, specific_pair, msg, compute_hour_signal=True,
                                  day_low_dict[key_day_ts],
                                  day_close_dict[key_day_ts], day_volu_dict[key_day_ts]]
             day_new_data = np.array(day_new_data_lst)
-            #print('new daily data')
-            #print(day_new_data)
+            print(f'new daily data added for pair {specific_pair}')
+            print(day_new_data)
             if fetcher.day_np.get(specific_pair) is None:
                 fetcher.day_np[specific_pair] = day_new_data
             else:
@@ -265,7 +266,7 @@ def pair_handling_message(fetcher, specific_pair, msg, compute_hour_signal=True,
 
             if compute_day_signal:
                 print('updating bot position for pair '+ str(specific_pair) + ' and signal '+str(ia_day_signal))
-#            fetcher.bot.update_position(specific_pair, ia_day_signal)
+
 
             if fetcher.hour_np.get(specific_pair) is not None and len(fetcher.hour_np[specific_pair])>0:
                 print('persisting hours dataframe for the previous day')
@@ -310,20 +311,29 @@ def pair_handling_message(fetcher, specific_pair, msg, compute_hour_signal=True,
             hour_new_data_lst = [key_hour_ts, hour_open_dict[key_hour_ts], hour_high_dict[key_hour_ts], hour_low_dict[key_hour_ts],
                             hour_close_dict[key_hour_ts], hour_volu_dict[key_hour_ts]]
             hour_new_data = np.array(hour_new_data_lst)
-            #print('new hour data')
-            #print(hour_new_data)
+            print(f'new hourly data added for pair {specific_pair}')
+            print(hour_new_data)
+
             if fetcher.hour_np.get(specific_pair) is None:
                 fetcher.hour_np[specific_pair] = hour_new_data
             else:
                 fetcher.hour_np[specific_pair] = np.vstack((fetcher.hour_np[specific_pair], hour_new_data))
 
             if compute_hour_signal:
-                ia_hour_signal = fetcher.launchDegenerateHourlySequential(specific_pair)
+                #            fetcher.bot.update_position(specific_pair, ia_day_signal)
+                fetcher.hour_signal_counter[trade_time_unix_timestamp_hour] = fetcher.hour_signal_counter[trade_time_unix_timestamp_hour] + 1
+                print(f'number of found signals for that hour {fetcher.hour_signal_counter[trade_time_unix_timestamp_hour]}')
+                fetcher.update_hour_dataframe(specific_pair)
+                if fetcher.hour_signal_counter[trade_time_unix_timestamp_hour]>=int(len(fetcher.hour_np)-0.1*len(fetcher.hour_np)):
+                    fetcher.launchAllCoinsHourlySignal()
+                    fetcher.hour_signal_counter[trade_time_unix_timestamp_hour] = np.nan
+                #ia_hour_signal = fetcher.launchDegenerateHourlySequential(specific_pair)
 
             if (len(fetcher.hour_np[specific_pair].shape)>1 and len(fetcher.hour_np[specific_pair])>3):
                 #print('last three hours')
                 #print(fetcher.hour_np[-3:,:])
-                print('hourly ia signal ' + str(ia_hour_signal))
+                #print('hourly ia signal ' + str(ia_hour_signal))
+                print('hourly signal')
             #fetcher.minute_df.loc[len(fetcher.minute_df)] = new_data
             if fetcher.minute_np.get(specific_pair) is not None and len(fetcher.minute_np[specific_pair])>0:
                 print('persisting minutes dataframe for the previous hour')
@@ -390,6 +400,7 @@ class RealTimeDegenerateBotLauncher():
         self.BINANCE_PUBLIC = BINANCE_PUBLIC
         self.BINANCE_SECRET = BINANCE_SECRET
 
+
         self.bot = binance_spot_bot.NaPoleonBinanceSpotBot(BINANCE_PUBLIC, BINANCE_SECRET)
 
         self.dropbox_backup =dropbox_backup
@@ -499,6 +510,8 @@ class RealTimeDegenerateBotLauncher():
         self.equi=False
         self.lookback_window = 25
         self.backtest=backtest
+        self.hour_signal_counter = defaultdict(lambda: 0.)
+
 
         if self.load_daily_histo and not self.backtest:
             print('reloading historical data')
@@ -1153,28 +1166,30 @@ class RealTimeDegenerateBotLauncher():
 
 
     def start_fetching_pair(self):
-        for me_pair in self.shitcoin_universe_pairs:
+        for me_pair in self.shitcoin_universe_50_pairs:
             bm = BinanceSocketManager(self.client)
             pair_callback = partial(pair_handling_message, self, me_pair)
             bm.start_aggtrade_socket(me_pair, pair_callback)
             bm.start()
 
-    def reset_minute_np_to_lookback_window(self, pair, loolback_window = 70):
-        print('minutes shape before hour reset')
-        print(self.minute_np[pair].shape)
-        print(self.minute_signal_np[pair].shape)
-        self.minute_np[pair] = self.minute_np[pair][-loolback_window:]
-        self.minute_signal_np[pair] = self.minute_signal_np[pair][-loolback_window:]
-        print('minutes shape afteer hour reset')
-        print(self.minute_np[pair].shape)
-        print(self.minute_signal_np[pair].shape)
+    def reset_minute_np_to_lookback_window(self, pair, loolback_window = 120):
+        if pair in self.minute_signal_np.keys():
+            print('minutes shape before hour reset')
+            print(self.minute_np[pair].shape)
+            print(self.minute_signal_np[pair].shape)
+            self.minute_np[pair] = self.minute_np[pair][-loolback_window:]
+            self.minute_signal_np[pair] = self.minute_signal_np[pair][-loolback_window:]
+            print('minutes shape afteer hour reset')
+            print(self.minute_np[pair].shape)
+            print(self.minute_signal_np[pair].shape)
 
-    def reset_hour_np_to_lookback_window(self, loolback_window = 70):
-        print('hours shape before day reset')
-        print(self.hour_np.shape)
-        self.hour_np = self.hour_np[-loolback_window:]
-        print('hours shape after day reset')
-        print(self.hour_np.shape)
+    def reset_hour_np_to_lookback_window(self, pair, loolback_window = 70):
+        if pair in self.hour_np.keys():
+            print('hours shape before day reset')
+            print(self.hour_np[pair].shape)
+            self.hour_np[pair] = self.hour_np[pair][-loolback_window:]
+            print('hours shape after day reset')
+            print(self.hour_np[pair].shape)
 
     def update_day_dataframe(self):
         print('updating day df')
@@ -1218,31 +1233,33 @@ class RealTimeDegenerateBotLauncher():
     def update_signal_minute_dataframe(self,pair):
         print('updating 60 minutes hour df')
         me_columns = self.generic_columns + [pair]
-        updated_df=pd.DataFrame(data=self.minute_signal_np[pair], columns = me_columns, index=range(len(self.minute_signal_np[pair])))
-        #updated_df['ts']=updated_df['ts'].astype(int)
-        # updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
-        # updated_df = updated_df.astype({'hour_ts': 'int64'})
-        # if updated_df['hour_ts'].nunique()>1:
-        #     print('too many hours saved together : investigate')
-        # hour_timestamp = updated_df['hour_ts'].max()
+        if pair in self.minute_signal_np.keys():
+            updated_df=pd.DataFrame(data=self.minute_signal_np[pair], columns = me_columns, index=range(len(self.minute_signal_np[pair])))
+            #updated_df['ts']=updated_df['ts'].astype(int)
+            # updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
+            # updated_df = updated_df.astype({'hour_ts': 'int64'})
+            # if updated_df['hour_ts'].nunique()>1:
+            #     print('too many hours saved together : investigate')
+            # hour_timestamp = updated_df['hour_ts'].max()
 
-        updated_df['ts'] = updated_df['ts'].astype(int)
-        me_ts =updated_df['ts']
-        me_hour_ts = me_ts-me_ts%3600
-        me_hour_ts = me_hour_ts.astype({'ts': 'int64'})
-        if me_hour_ts.nunique()>1:
-            print('too many days saved together : investigate')
-        hour_timestamp = me_hour_ts.max()
+            updated_df['ts'] = updated_df['ts'].astype(int)
+            me_ts =updated_df['ts']
+            me_hour_ts = me_ts-me_ts%3600
+            me_hour_ts = me_hour_ts.astype({'ts': 'int64'})
+            if me_hour_ts.nunique()>1:
+                print('too many days saved together : investigate')
+            hour_timestamp = me_hour_ts.max()
 
-        hour_dt_object = datetime.datetime.utcfromtimestamp(hour_timestamp)
-        filename = pair+'_'+hour_dt_object.strftime('%d_%b_%Y_%H') #str(hourr_timestamp)
-        updated_df['ts'] = pd.to_datetime(updated_df['ts'], unit='s')
-        updated_df = updated_df.sort_values(by=['ts'])
-        updated_df = updated_df.rename(columns={"ts": "date"}, errors="raise")
-        updated_df = updated_df.set_index(updated_df['date'])
-        updated_df = updated_df.drop(columns=['date'])
-        self.minute_signal_df = updated_df.copy()
-        return filename
+            hour_dt_object = datetime.datetime.utcfromtimestamp(hour_timestamp)
+            filename = pair+'_'+hour_dt_object.strftime('%d_%b_%Y_%H') #str(hourr_timestamp)
+            updated_df['ts'] = pd.to_datetime(updated_df['ts'], unit='s')
+            updated_df = updated_df.sort_values(by=['ts'])
+            updated_df = updated_df.rename(columns={"ts": "date"}, errors="raise")
+            updated_df = updated_df.set_index(updated_df['date'])
+            updated_df = updated_df.drop(columns=['date'])
+            self.minute_signal_df[pair] = updated_df.copy()
+            return filename
+        return None
 
     def update_minute_dataframe(self):
         print('updating minute df')
@@ -1261,39 +1278,39 @@ class RealTimeDegenerateBotLauncher():
 
     def update_hour_dataframe(self, pair):
         print('updating minute df')
-        if len(self.hour_np[pair].shape)>1:
-            updated_df=pd.DataFrame(data=self.hour_np[pair], columns = self.generic_columns, index=range(len(self.hour_np[pair])))
-        else:
-            updated_df=pd.DataFrame(data=self.hour_np[pair].reshape(1,len(self.hour_np[pair])), columns = self.generic_columns, index=[0])
+        if pair in self.hour_np.keys() :
+            if len(self.hour_np[pair].shape)>1:
+                updated_df=pd.DataFrame(data=self.hour_np[pair], columns = self.generic_columns, index=range(len(self.hour_np[pair])))
+            else:
+                updated_df=pd.DataFrame(data=self.hour_np[pair].reshape(1,len(self.hour_np[pair])), columns = self.generic_columns, index=[0])
 
-        # updated_df = updated_df.astype(
-        #     {'ts': 'float64', 'open': 'float64', 'high': 'float64', 'low': 'float64', 'close': 'float64','volumefrom': 'float64'}
-        #                                )
-        # updated_df['ts']=updated_df['ts'].astype(int)
-        # updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
-        # updated_df['day_ts'] = updated_df['ts']-updated_df['ts']%(3600*24)
-        # updated_df = updated_df.astype({'hour_ts': 'int64'})
-        # updated_df = updated_df.astype({'day_ts': 'int64'})
-        # if updated_df['day_ts'].nunique()>1:
-        #     print('too many days saved together : investigate')
-        # day_timestamp = updated_df['day_ts'].max()
+            # updated_df = updated_df.astype(
+            #     {'ts': 'float64', 'open': 'float64', 'high': 'float64', 'low': 'float64', 'close': 'float64','volumefrom': 'float64'}
+            #                                )
+            # updated_df['ts']=updated_df['ts'].astype(int)
+            # updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
+            # updated_df['day_ts'] = updated_df['ts']-updated_df['ts']%(3600*24)
+            # updated_df = updated_df.astype({'hour_ts': 'int64'})
+            # updated_df = updated_df.astype({'day_ts': 'int64'})
+            # if updated_df['day_ts'].nunique()>1:
+            #     print('too many days saved together : investigate')
+            # day_timestamp = updated_df['day_ts'].max()
 
-        updated_df['ts'] = updated_df['ts'].astype(int)
-        me_ts =updated_df['ts']
-        me_day_ts = me_ts-me_ts%(3600*24)
-        me_day_ts = me_day_ts.astype({'ts': 'int64'})
-        if me_day_ts.nunique()>1:
-            print('too many days saved together : investigate')
-        day_timestamp = me_day_ts.max()
+            updated_df['ts'] = updated_df['ts'].astype(int)
+            me_ts =updated_df['ts']
+            me_day_ts = me_ts-me_ts%(3600*24)
+            me_day_ts = me_day_ts.astype({'ts': 'int64'})
+            if me_day_ts.nunique()>1:
+                print('too many days saved together : investigate')
 
-        updated_df['ts'] = pd.to_datetime(updated_df['ts'], unit='s')
-        updated_df = updated_df.sort_values(by=['ts'])
-        updated_df = updated_df.rename(columns={"ts": "date"}, errors="raise")
-        updated_df = updated_df.set_index(updated_df['date'])
-        updated_df = updated_df.drop(columns=['date'])
+            updated_df['ts'] = pd.to_datetime(updated_df['ts'], unit='s')
+            updated_df = updated_df.sort_values(by=['ts'])
+            updated_df = updated_df.rename(columns={"ts": "date"}, errors="raise")
+            updated_df = updated_df.set_index(updated_df['date'])
+            updated_df = updated_df.drop(columns=['date'])
 
-        self.hour_df[pair] = updated_df.copy()
-        self.last_hour = max(self.hour_df[pair].index)
+            self.hour_df[pair] = updated_df.copy()
+            self.last_hour = max(self.hour_df[pair].index)
 
 
     def upload_last_day_24_hours_signals_dataframe(self):
@@ -1392,10 +1409,15 @@ class RealTimeDegenerateBotLauncher():
         new_day_sig_dic = {'ts': last_day_ts, 'day_discrete_sig': discrete_day_signal}
         self.on_the_fly_discrete_minute_signals.append(new_day_sig_dic)
 
+    def launchAllCoinsHourlySignal(self):
+        for me_pair in self.shitcoin_universe_50_pairs:
+            self.update_hour_dataframe(me_pair)
+
+        print('done')
+
     def launchDegenerateHourlySequential(self, pair):
         self.update_hour_dataframe(pair)
         print('launching sequential minute computation for alphas at '+str(self.last_hour))
-
         signal_value, trigger_value = self.runDegenerateOHourSignal(pair)
         run_results = []
         trigger_results = []
@@ -1631,6 +1653,23 @@ class RealTimeDegenerateBotLauncher():
         return hourly_df, last_day_df
 
 
+    def get_aggregated_live_data(self,):
+        aggregated_df = None
+
+        for me_pair in self.shitcoin_universe_pair:
+            print(f'fetching {me_pair}')
+            data_df = self.bot.get_hourly_ohlc(me_pair, starting_date, running_date)
+            print(f'saving {path}')
+
+            data_df = data_df.rename(columns={'close': me_pair})
+            if aggregated_df is None:
+                aggregated_df = data_df[[me_pair]].copy()
+            else:
+                aggregated_df = pd.merge(aggregated_df.copy(), data_df[[me_pair]].copy(), how='left',
+                                         left_index=True, right_index=True)
+        return aggregated_df
+
+
     def get_aggregated_historical_data(self, starting_date, running_date, shitcoin_universe_pair, reaggregate = True, refetch_pricing = True, hourly_prices = True):
         starting_date_string = starting_date.strftime('%m-%d-%y')
         print('fetching universe')
@@ -1665,8 +1704,7 @@ class RealTimeDegenerateBotLauncher():
             aggregated_df.to_pickle(aggpath)
         return aggregated_df
 
-    def backtest_strategy(self, starting_date, running_date):
-
+    def backtest_strategy(self, starting_date, running_date, backtest_slippage_fees = 15e-4):
         maxPerf = -np.inf
         maxLookback = np.nan
         maxChosen = np.nan
@@ -1685,7 +1723,6 @@ class RealTimeDegenerateBotLauncher():
         maxMDDEqui = np.nan
         maxMDDUni = np.nan
 
-
         lookback_windows = [5, 10, 15, 20, 25, 30, 35, 40, 45, 50,75,100,150,200]
         nb_chosens = [1,2,3,4,5,10,15,20,25]
         equis = [True, False]
@@ -1695,15 +1732,10 @@ class RealTimeDegenerateBotLauncher():
         #maxPerf 71299304.29933749 maxLookback 20 maxChosen 3 maxEqui True
         #maxPerf 211476079.65838435 maxLookback 20 maxChosen 3 maxEqui True
         #maxPerf -0.1239545943932765 maxLookback 15 maxChosen 25 maxEqui True
-
-        ###### best sharpe and best calmar
-        #lookback_windows = [20]
-        #nb_chosens = [15]
-        #equis = [False]
-
-        #lookback_windows = [20]
-        #nb_chosens = [3]
-        #equis = [True]
+        lookback_windows = [20]
+        nb_chosens = [3]
+        universe_sizes = [50]
+        equis = [True]
 
         for universe_size in universe_sizes:
             for lookback_window in lookback_windows:
@@ -1748,8 +1780,6 @@ class RealTimeDegenerateBotLauncher():
                         # nb_chosen = 3
                         # equi = False
 
-
-
                         df_perf = data_df[shitcoin_universe].pct_change()
                         df_perf.columns = [f'R{col}' for col in df_perf.columns]
 
@@ -1762,23 +1792,22 @@ class RealTimeDegenerateBotLauncher():
                         print('signal computed')
                         print(signal_df.shape)
 
-                        signal_df.columns = [f'S{col}' for col in signal_df.columns]
+                        signal_df.columns = [f'NS{col}' for col in signal_df.columns]
+                        for me_col in shitcoin_universe:
+                            if f'NS{me_col}' not in signal_df.columns:
+                                signal_df[f'NS{me_col}'] = 0.
+                            signal_df[f'S{me_col}'] = signal_df[f'NS{me_col}'].shift(1)
+                            signal_df[f'TO{me_col}'] = abs( signal_df[f'S{me_col}'] -  signal_df[f'NS{me_col}'])
 
-                        shifted_signal_df = signal_df.shift(1)
+                        allin_df = pd.merge(df_perf.copy(), signal_df.copy(), left_index=True, right_index=True)
 
-                        allin_df = pd.merge(df_perf.copy(), shifted_signal_df, left_index=True, right_index=True)
                         allin_df = allin_df.fillna(0.)
                         print('done')
 
                         # dfperf = dfret[['BTC']].copy()
                         # dfperf['expo'] = expo(dfsig2['Counter'].values)
                         for i in shitcoin_universe:
-                            if f'S{i}' not in allin_df.columns:
-                                allin_df[f'S{i}'] = 0.
-
-                            allin_df['RS{}'.format(i)] = allin_df[f'R{i}'] * allin_df[f'S{i}']  # * np.append(np.zeros(1),
-                            # dfsig2['S{}'.format(i)].values[:-1]) * np.append(
-                        #        np.zeros(1), dfperf['expo'].values[:-1])
+                            allin_df['RS{}'.format(i)] = allin_df[f'R{i}'] * allin_df[f'S{i}'] - allin_df[f'TO{i}']*backtest_slippage_fees  # * np.append(np.zeros(1),
 
                         allin_df = allin_df.dropna()
                         allin_df['R'] = 0
@@ -1792,7 +1821,7 @@ class RealTimeDegenerateBotLauncher():
                         S = allin_df['R'].values
                         R = allin_df['RBTCUSDT'].values
 
-                        replot = False
+                        replot = True
                         if replot:
                             f, ax = plt.subplots(1, 1, figsize=(16, 8))
                             ax.plot(allin_df.index, np.cumprod(1 + S), linewidth=2, label="Smart Beta Vs USDT")
