@@ -109,12 +109,25 @@ def plot_sig_vs_ret(data_df, return_col = 'close', sig_col = None):
     return fig
 
 
-def plot_scatter_indicator_vs_ret(data_df, return_col = 'close', sig_col = None):
+def plot_scatter_indicator_vs_ret(data_df, return_col = 'close', sig_col = None, color_red_return = True):
+    if color_red_return:
+        data_df['color']='red'
+        data_df['to_filter'] = data_df[return_col] >= 0.
+        data_df['color'][ data_df['to_filter']] = 'green'
+        # fig = px.scatter(data_df, x=sig_col, y=return_col,color='color')
+
+
+        fig = go.Figure(data=go.Scatter(
+            x=data_df[sig_col],
+            y=data_df[return_col],
+            mode='markers',
+            marker=dict(color=list(data_df['color']))
+        ))
+
+        return fig
     fig = go.Figure(data=go.Scatter(x=data_df[sig_col], y=data_df[return_col], mode='markers'))
     return fig
 
-
-
 def plot_box_plot_sig_vs_ret(data_df, return_col = 'close', sig_col = None):
     fig = px.box(data_df, x=sig_col, y=return_col, points="all")
     return fig
