@@ -180,5 +180,5 @@ class _NapoleonLoggerManager(_LoggerManager):
             return super(_NapoleonLoggerManager, self).get_logger()
 
 
-NapoleonLoggerManager = _NapoleonLoggerManager
-log = _NapoleonLoggerManager().get_logger(__package__)
+#NapoleonLoggerManager = _NapoleonLoggerManager
+#log = _NapoleonLoggerManager().get_logger(__package__)
